@@ -21,7 +21,7 @@ ASpellVersion=${ASpellVersion:-"0.60.6.1"}
 ASpellSource="aspell-${ASpellVersion}"
 ASpellLibrary="libaspell.15.dylib"
 
-HunSpellVersion=${HunSpellVersion:-"1.3.2"}
+HunSpellVersion=${HunSpellVersion:-"1.3.3"}
 HunSpellSource="hunspell-${HunSpellVersion}"
 HunSpellLibrary="libhunspell-1.3.0.dylib"
 
@@ -65,7 +65,7 @@ case "${QtVersion}:${QtAPI}" in
 	QtConfigureOptions="${QtConfigureOptions} -nomake examples -nomake demos -nomake docs -nomake tools"
 	QtMajorVersion=qt5
 	;;
-5.6*)
+5.6*|5.7*)
 	QtConfigureOptions="${QtConfigureOptions} -no-strip"
 	QtConfigureOptions="${QtConfigureOptions} -no-kms -no-pkg-config"
 	QtConfigureOptions="${QtConfigureOptions} -nomake examples -nomake tools"
@@ -276,6 +276,11 @@ while [ $# -gt 0 ]; do
 		LyxOnlyPackage=$(echo ${1}|cut -d= -f2)
 		shift
 		;;
+	--enable-cxx11)
+		LyXConfigureOptions="${LyXConfigureOptions} ${1}"
+		EnableCXX11="--enable-cxx11"
+		shift
+		;;
 	--*)
 		LyXConfigureOptions="${LyXConfigureOptions} ${1}"
 		shift
@@ -350,12 +355,8 @@ case "${QtVersion}" in
 	QtLibraries="QtSvg QtXml QtPrintSupport QtWidgets QtGui QtNetwork QtConcurrent QtCore"
 	QtFrameworkVersion="5"
 	;;
-5.2.*|5.3.*|5.4.*)
-	QtLibraries="QtSvg QtXml QtPrintSupport QtMacExtras QtWidgets QtGui QtNetwork QtConcurrent QtCore"
-	QtFrameworkVersion="5"
-	;;
 5*)
-	QtLibraries="QtSvg QtXml QtPrintSupport QtDBus QtMacExtras QtWidgets QtGui QtNetwork QtConcurrent QtCore"
+	QtLibraries="QtSvg QtXml QtPrintSupport QtMacExtras QtWidgets QtGui QtNetwork QtConcurrent QtCore"
 	QtFrameworkVersion="5"
 	;;
 *)
@@ -621,7 +622,7 @@ framework_name() {
 	echo "Frameworks/${1}.framework"
 }
 
-LYX_FILE_LIST="lyx lyxclient tex2lyx"
+LYX_FILE_LIST="lyx lyxclient tex2lyx lyxconvert"
 BUNDLE_PATH="Contents/MacOS"
 LYX_BUNDLE_PATH="${LyxAppPrefix}/${BUNDLE_PATH}"
 build_lyx() {
@@ -629,6 +630,14 @@ build_lyx() {
 	if [ -n "${LyxAppZip}" -a -f "${LyxAppZip}" ]; then rm "${LyxAppZip}"; fi
 	if [ -d "${LyxAppPrefix}" ]; then rm -rf "${LyxAppPrefix}"; fi
 
+	case "${EnableCXX11}" in
+	"--enable-cxx11")
+		export CC=cc
+		export CXX="c++ -stdlib=libc++"
+		export CXXFLAGS=-std=c++11
+		;;
+	esac
+
 	# -------------------------------------
 	# Automate configure check
 	# -------------------------------------
@@ -695,7 +704,7 @@ build_lyx() {
 				mv "${LYX_BUNDLE_PATH}/${file}"\
 					"${LYX_BUNDLE_PATH}/${file}-${arch}" 
 			else
-				echo ERROR: Cannot build and install LyX for ${arch}.
+				echo ERROR: Cannot build and install ${file} for ${arch}.
 				exit 1
 			fi
 		done
@@ -758,6 +767,7 @@ EOF
 		test -d "${condir}/${fwdir}" || (
 			echo Copy framework "${source}/lib/"$(basename "${fwdir}")
 			cp -pR "${source}/lib/"$(basename "${fwdir}") "${condir}/${fwdir}"
+			rm -f "${condir}/${fwdir}/${version}${libnm}"_debug
 			installname -id "@executable_path/../${fwdir}/${version}${libnm}" "${condir}/${fwdir}/${version}${libnm}"
 			find "${condir}/PlugIns" "${condir}/"$(dirname "${fwdir}") -name Headers -prune -o -type f -print | while read filename ; do
 				if [ "${filename}" != "${target}" ]; then
@@ -783,11 +793,13 @@ EOF
 			"@executable_path/../${fwdir}/${version}${libnm}"\
 			"${target}"
 	done
-	if [ ! -d "${condir}/translations" ]; then
-		mkdir -p "${condir}/translations"
+	if [ -d "${source}"/translations ]; then
+		if [ ! -d "${condir}/translations" ]; then
+			mkdir -p "${condir}/translations"
+		fi
+		echo Copy Qt translations to "${condir}/translations"
+		cp -p "${source}"/translations/qt_*.qm "${condir}/translations"
 	fi
-	echo Copy Qt translations to "${condir}/translations"
-	cp -p "${source}"/translations/qt_*.qm "${condir}/translations"
 }
 
 # -------------------------
@@ -931,28 +943,27 @@ make_dmg() {
 	test -d /Volumes/"${LyxBase}" && rmdir /Volumes/"${LyxBase}"
 
 	# Mount the disk image
-	hdiutil attach "${DMGNAME}.sparseimage"
+	DEVICES=$(hdiutil attach "${DMGNAME}.sparseimage" | cut -f 1)
 
 	# Obtain device information
-	DEVS=$(hdiutil attach "${DMGNAME}.sparseimage" | cut -f 1)
-	DEV=$(echo $DEVS | cut -f 1 -d ' ')
-	VOLUME=$(mount |grep ${DEV} | cut -f 3 -d ' ')
+	DEVICE=$(echo $DEVICES | cut -f 1 -d ' ')
+	VOLUME=$(mount |grep ${DEVICE} | cut -f 3 -d ' ')
 
 	# copy in the application bundle
-	cp -Rp "${LyxAppDir}.app" "${VOLUME}/${LyxName}.app"
+	ditto --hfsCompression "${LyxAppDir}.app" "${VOLUME}/${LyxName}.app"
 
 	# copy in background image
 	mkdir -p "${VOLUME}/Pictures"
-	cp "${DmgBackground}" "${VOLUME}/Pictures/background.png"
+	ditto --hfsCompression "${DmgBackground}" "${VOLUME}/Pictures/background.png"
 	# symlink applications
 	ln -s /Applications/ "${VOLUME}"/Applications
-	test -d "${DocumentationDir}" && cp -r "${DocumentationDir}" "${VOLUME}"
+	test -d "${DocumentationDir}" && ditto --hfsCompression "${DocumentationDir}" "${VOLUME}"
 	set_bundle_display_options "${VOLUME}" ${BG_W} ${BG_H}
 	${XCODE_DEVELOPER}/Tools/SetFile -a C "${VOLUME}"
 	mv "${VOLUME}/Pictures" "${VOLUME}/.Pictures"
 
 	# Unmount the disk image
-	hdiutil detach ${DEV}
+	hdiutil detach ${DEVICE}
 
 	# Convert the disk image to read-only
 	hdiutil convert "${DMGNAME}.sparseimage" -format UDBZ -o "${DMGNAME}.dmg"
