@@ -520,6 +520,8 @@ public:
 	/// \return the bibliography information for this buffer's master,
 	/// or just for it, if it isn't a child.
 	BiblioInfo const & masterBibInfo() const;
+	/// \return this buffer's bibliography information
+	BiblioInfo const & bibInfo() const;
 	/// collect bibliography info from the various insets in this buffer.
 	void collectBibKeys(support::FileNameList &) const;
 	/// add some BiblioInfo to our cache
@@ -529,6 +531,8 @@ public:
 	///
 	void makeCitationLabels() const;
 	///
+	void invalidateCiteLabels() const;
+	///
 	bool citeLabelsValid() const;
 	///
 	void getLabelList(std::vector<docstring> &) const;
