@@ -40,6 +40,7 @@
 
 #include "insets/InsetBibitem.h"
 #include "insets/InsetBranch.h"
+#include "insets/InsetCitation.h"
 #include "insets/InsetCommand.h"
 #include "insets/InsetFlex.h"
 #include "insets/InsetGraphics.h"
@@ -360,6 +361,16 @@ pasteSelectionHelper(DocIterator const & cur, ParagraphList const & parlist,
 			break;
 		}
 
+		case CITE_CODE: {
+			InsetCitation & cit = static_cast<InsetCitation &>(*it);
+			// This actually only needs to be done if the cite engine
+			// differs, but we do it in general.
+			cit.redoLabel();
+			// We need to update the list of citations.
+			need_update = true;
+			break;
+		}
+
 		case BIBITEM_CODE: {
 			// check for duplicates
 			InsetBibitem & bib = static_cast<InsetBibitem &>(*it);
@@ -808,11 +819,11 @@ void switchBetweenClasses(DocumentClassConstPtr oldone,
 
 	// character styles and hidden table cells
 	InsetIterator const i_end = inset_iterator_end(in);
-	for (InsetIterator it = inset_iterator_begin(in); it != i_end; ++it) {
-		InsetCode const code = it->lyxCode();
+	for (InsetIterator iit = inset_iterator_begin(in); iit != i_end; ++iit) {
+		InsetCode const code = iit->lyxCode();
 		if (code == FLEX_CODE) {
 			// FIXME: Should we verify all InsetCollapsible?
-			docstring const layoutName = it->layoutName();
+			docstring const layoutName = iit->layoutName();
 			docstring const & n = newone->insetLayout(layoutName).name();
 			bool const is_undefined = n.empty() ||
 				n == DocumentClass::plainInsetLayout().name();
@@ -833,14 +844,14 @@ void switchBetweenClasses(DocumentClassConstPtr oldone,
 			// To warn the user that something had to be done.
 			errorlist.push_back(ErrorItem(
 			                              _("Undefined flex inset"), s,
-			                              {it.paragraph().id(), it.pos()},
-			                              {it.paragraph().id(), it.pos()+1}));
+						      {iit.paragraph().id(), iit.pos()},
+						      {iit.paragraph().id(), iit.pos() + 1}));
 		} else if (code == TABULAR_CODE) {
 			// The recursion above does not catch paragraphs in "hidden" cells,
 			// i.e., ones that are part of a multirow or multicolum. So we need
 			// to handle those separately.
 			// This is the cause of bug #9049.
-			InsetTabular * table = it->asInsetTabular();
+			InsetTabular * table = iit->asInsetTabular();
 			table->setLayoutForHiddenCells(newtc);
 		}
 	}
@@ -1040,7 +1051,10 @@ void copySelectionToStack(Cursor const & cur, CutStack & cutstack)
 		BufferParams const & bp = cur.buffer()->params();
 		// FIXME This should be the plain layout...right?
 		par.setLayout(bp.documentClass().plainLayout());
-		par.insert(0, grabSelection(cur), Font(), Change(Change::UNCHANGED));
+		// For pasting into text, we set the language to the paragraph language
+		// (rather than the default_language which is always English; see #2596)
+		par.insert(0, grabSelection(cur), Font(sane_font, par.getParLanguage(bp)),
+			   Change(Change::UNCHANGED));
 		pars.push_back(par);
 		cutstack.push(make_pair(pars, bp.documentClassPtr()));
 	}
@@ -1056,6 +1070,12 @@ void copySelectionToStack()
 }
 
 
+void copySelectionToTemp(Cursor & cur)
+{
+	copySelectionToStack(cur, tempCut);
+}
+
+
 void copySelection(Cursor const & cur, docstring const & plaintext)
 {
 	// In tablemode, because copy and paste actually use special table stack
@@ -1228,6 +1248,10 @@ bool pasteClipboardText(Cursor & cur, ErrorList & errorList, bool asParagraphs,
 				if (available)
 					available = !buffer.paragraphs().empty();
 				if (available && !buffer.paragraphs()[0].empty()) {
+					// TeX2lyx (also used in the HTML chain) assumes English as document language
+					// if no language is explicitly set (as is the case here).
+					// We thus reset the temp buffer's language to the context language
+					buffer.changeLanguage(buffer.language(), cur.getFont().language());
 					cur.recordUndo();
 					pasteParagraphList(cur, buffer.paragraphs(),
 						buffer.params().documentClassPtr(), errorList);
