@@ -25,8 +25,9 @@
 
 #include <boost/cstdint.hpp>
 
-#include <sstream>
+#include <iterator>
 #include <algorithm>
+#include <sstream>
 
 using namespace std;
 using namespace lyx::support;
@@ -58,7 +59,7 @@ MathAlphaSet mathalpha;
 /// The highest code point in UCS4 encoding (1<<20 + 1<<16)
 char_type const max_ucs4 = 0x110000;
 
-} // namespace anon
+} // namespace
 
 
 EncodingException::EncodingException(char_type c)
@@ -259,17 +260,17 @@ vector<char_type> Encoding::symbolsList() const
 	// assure the used encoding is properly initialized
 	init();
 
-	// first all encodable characters
-	vector<char_type> symbols(encodable_.begin(), encodable_.end());
-	// add those below start_encodable_
+	// first all those below start_encodable_
+	vector<char_type> symbols;
 	for (char_type c = 0; c < start_encodable_; ++c)
 		symbols.push_back(c);
+	// add all encodable characters
+	copy(encodable_.begin(), encodable_.end(), back_inserter(symbols));
 	// now the ones from the unicodesymbols file that are not already there
-	CharInfoMap::const_iterator const end = unicodesymbols.end();
-	CharInfoMap::const_iterator it = unicodesymbols.begin();
-	for (; it != end; ++it)
-		if (find(symbols.begin(), symbols.end(), it->first) == symbols.end())
-			symbols.push_back(it->first);
+	for (pair<char_type, CharInfo> const & elem : unicodesymbols) {
+		if (find(symbols.begin(), symbols.end(), elem.first) == symbols.end())
+			symbols.push_back(elem.first);
+	}
 	// finally, sort the vector
 	sort(symbols.begin(), symbols.end());
 	return symbols;
@@ -604,6 +605,16 @@ bool Encodings::isMathAlpha(char_type c)
 }
 
 
+bool Encodings::isUnicodeTextOnly(char_type c)
+{
+	if (isASCII(c) || isMathAlpha(c))
+		return false;
+
+	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
+	return it == unicodesymbols.end() || it->second.mathcommand().empty();
+}
+
+
 Encoding const *
 Encodings::fromLyXName(string const & name, bool allowUnsafe) const
 {
