@@ -358,7 +358,8 @@ public:
 	///
 	char_type getChar(pos_type pos) const;
 	/// Get the char, but mirror all bracket characters if it is right-to-left
-	char_type getUChar(BufferParams const &, pos_type pos) const;
+	char_type getUChar(BufferParams const &, OutputParams const &,
+			   pos_type pos) const;
 	/// pos <= size() (there is a dummy font change at the end of each par)
 	void setFont(pos_type pos, Font const & font);
 	///
@@ -505,6 +506,10 @@ public:
 	/// presently used only in the XHTML output routines.
 	std::string magicLabel() const;
 
+	/// anonymizes the paragraph contents (but not the paragraphs
+	/// contained inside it. Does not handle undo.
+	void anonymize();
+
 private:
 	/// Expand the counters for the labelstring of \c layout
 	docstring expandParagraphLabel(Layout const &, BufferParams const &,
