@@ -34,8 +34,10 @@ string const sec_bookmarks = "[bookmarks]";
 string const sec_session = "[session info]";
 string const sec_toolbars = "[toolbars]";
 string const sec_lastcommands = "[last commands]";
+string const sec_authfiles = "[auth files]";
+string const sec_shellescape = "[shell escape files]";
 
-} // anon namespace
+} // namespace
 
 
 namespace lyx {
@@ -128,7 +130,7 @@ void LastOpenedSection::read(istream & is)
 				lof.file_name = file;
 				lastopened.push_back(lof);
 			} else {
-				LYXERR(Debug::INIT, 
+				LYXERR(Debug::INIT,
 					"LyX: Warning: Ignore last opened file: " << tmp);
 			}
 		} catch (...) {
@@ -336,7 +338,7 @@ LastCommandsSection::LastCommandsSection(unsigned int num) :
 	setNumberOfLastCommands(num);
 }
 
-	
+
 void LastCommandsSection::read(istream & is)
 {
 	string tmp;
@@ -419,6 +421,10 @@ void Session::readFile()
 			bookmarks().read(is);
 		else if (tmp == sec_lastcommands)
 			lastCommands().read(is);
+		else if (tmp == sec_authfiles)
+			authFiles().read(is);
+		else if (tmp == sec_shellescape)
+			shellescapeFiles().read(is);
 
 		else
 			LYXERR(Debug::INIT, "LyX: Warning: unknown Session section: " << tmp);
@@ -438,9 +444,136 @@ void Session::writeFile() const
 		lastFilePos().write(os);
 		lastCommands().write(os);
 		bookmarks().write(os);
+		authFiles().write(os);
+		shellescapeFiles().write(os);
 	} else
 		LYXERR(Debug::INIT, "LyX: Warning: unable to save Session: "
 		       << session_file);
 }
 
+
+AuthFilesSection::AuthFilesSection() {  }
+
+
+void AuthFilesSection::read(istream & is)
+{
+	string tmp;
+	do {
+		char c = is.peek();
+		if (c == '[')
+			break;
+		getline(is, tmp);
+		if (tmp.empty() || tmp[0] == '#' || tmp[0] == ' ' || !FileName::isAbsolute(tmp))
+			continue;
+
+		// read lastfiles
+		FileName const file(tmp);
+		if (file.exists() && !file.isDirectory())
+			auth_files_.insert(tmp);
+		else
+			LYXERR(Debug::INIT, "LyX: Warning: Ignore auth file: " << tmp);
+	} while (is.good());
 }
+
+
+void AuthFilesSection::write(ostream & os) const
+{
+	os << '\n' << sec_authfiles << '\n';
+	copy(auth_files_.begin(), auth_files_.end(),
+	     ostream_iterator<std::string>(os, "\n"));
+}
+
+
+bool AuthFilesSection::find(string const & name) const
+{
+	if (auth_files_.find(name) != auth_files_.end())
+		return true;
+
+	return false;
+}
+
+
+void AuthFilesSection::insert(string const & name)
+{
+	auth_files_.insert(name);
+}
+
+
+void ShellEscapeSection::read(istream & is)
+{
+	string s;
+	do {
+		char c = is.peek();
+		if (c == '[')
+			break;
+		getline(is, s);
+		if (s.empty() || s[0] == '#' || s[0] == ' ' || !FileName::isAbsolute(s))
+			continue;
+
+		// read shellescape files
+		FileName const file(s.substr(0, s.length() - 2));
+		if (file.exists() && !file.isDirectory())
+			shellescape_files_.insert(s);
+		else
+			LYXERR(Debug::INIT, "LyX: Warning: Ignore shellescape file: " << file);
+	} while (is.good());
+}
+
+
+void ShellEscapeSection::write(ostream & os) const
+{
+	os << '\n' << sec_shellescape << '\n';
+	copy(shellescape_files_.begin(), shellescape_files_.end(),
+	     ostream_iterator<std::string>(os, "\n"));
+}
+
+
+bool ShellEscapeSection::find(string const & name) const
+{
+	if (shellescape_files_.find(name + ",0") != shellescape_files_.end())
+		return true;
+
+	return findAuth(name);
+}
+
+
+bool ShellEscapeSection::findAuth(string const & name) const
+{
+	if (shellescape_files_.find(name + ",1") != shellescape_files_.end())
+		return true;
+
+	return false;
+}
+
+
+void ShellEscapeSection::insert(string const & name, bool auth)
+{
+	set<string>::iterator it;
+	string const name0 = name + ",0";
+	string const name1 = name + ",1";
+
+	if (auth) {
+		it = shellescape_files_.find(name0);
+		if (it != shellescape_files_.end())
+			shellescape_files_.erase(it);
+		shellescape_files_.insert(name1);
+	} else {
+		it = shellescape_files_.find(name1);
+		if (it != shellescape_files_.end())
+			shellescape_files_.erase(it);
+		shellescape_files_.insert(name0);
+	}
+}
+
+
+void ShellEscapeSection::remove(string const & name)
+{
+	set<string>::iterator it = shellescape_files_.find(name + ",0");
+	if (it == shellescape_files_.end())
+		it = shellescape_files_.find(name + ",1");
+	if (it != shellescape_files_.end())
+		shellescape_files_.erase(it);
+}
+
+
+} // namespace lyx
