@@ -61,7 +61,7 @@ public:
 	int docbook(odocstream &, OutputParams const &) const;
 	///
 	docstring xhtml(XHTMLStream &, OutputParams const &) const;
-	/// 
+	///
 	void toString(odocstream &) const;
 	///
 	void forOutliner(docstring &, size_t const, bool const) const;
@@ -71,7 +71,7 @@ public:
 	void updateBuffer(ParIterator const & it, UpdateType);
 	///
 	void addToToc(DocIterator const & di, bool output_active,
-				  UpdateType utype) const;
+				  UpdateType utype, TocBackend & backend) const;
 	///
 	bool forceLTR() const { return true; }
 	//@}
@@ -86,13 +86,6 @@ public:
 	static bool isCompatibleCommand(std::string const & s);
 	//@}
 
-	//FIXME: private
-	/// \name Private functions inherited from InsetCommand class
-	//@{
-	///
-	docstring screenLabel() const { return screen_label_; }
-	//@}
-
 protected:
 	///
 	InsetRef(InsetRef const &);
@@ -103,7 +96,13 @@ private:
 	///
 	Inset * clone() const { return new InsetRef(*this); }
 	//@}
-	
+
+	/// \name Private functions inherited from InsetCommand class
+	//@{
+	///
+	docstring screenLabel() const;
+	//@}
+
 	/// \return the label with things that need to be escaped escaped
 	docstring getEscapedLabel(OutputParams const &) const;
 	/// \return the command for a formatted reference to ref
@@ -111,11 +110,13 @@ private:
 	/// \param argument for reference command
 	/// \param prefix of the label (before :)
 	docstring getFormattedCmd(docstring const & ref, docstring & label,
-			docstring & prefix) const;
+			docstring & prefix, docstring const & caps) const;
 
 	///
 	mutable docstring screen_label_;
 	///
+	mutable bool broken_;
+	///
 	mutable docstring tooltip_;
 };
 
