@@ -11,11 +11,14 @@
 #include <config.h>
 
 #include "InsetMathUnderset.h"
+
 #include "MathData.h"
 #include "MathStream.h"
 
 #include "Cursor.h"
 #include "LaTeXFeatures.h"
+#include "MetricsInfo.h"
+
 
 using namespace std;
 
@@ -29,29 +32,29 @@ Inset * InsetMathUnderset::clone() const
 
 void InsetMathUnderset::metrics(MetricsInfo & mi, Dimension & dim) const
 {
+	Changer dummy2 = mi.base.changeEnsureMath();
 	Dimension dim1;
 	cell(1).metrics(mi, dim1);
-	FracChanger dummy(mi.base);
+	Changer dummy = mi.base.changeFrac();
 	Dimension dim0;
 	cell(0).metrics(mi, dim0);
 	dim.wid = max(dim0.width(), dim1.width()) + 4;
 	dim.asc = dim1.ascent();
 	dim.des = dim1.descent() + dim0.height() + 4;
-	metricsMarkers(dim);
 }
 
 
 void InsetMathUnderset::draw(PainterInfo & pi, int x, int y) const
 {
+	Changer dummy2 = pi.base.changeEnsureMath();
 	Dimension const dim = dimension(*pi.base.bv);
 	Dimension const & dim0 = cell(0).dimension(*pi.base.bv);
 	Dimension const & dim1 = cell(1).dimension(*pi.base.bv);
 	int m  = x + dim.wid / 2;
 	int yo = y + dim1.descent() + dim0.ascent() + 1;
 	cell(1).draw(pi, m - dim1.width() / 2, y);
-	FracChanger dummy(pi.base);
+	Changer dummy = pi.base.changeFrac();
 	cell(0).draw(pi, m - dim0.width() / 2, yo);
-	drawMarkers(pi, x, y);
 }
 
 
